@@ -25,6 +25,13 @@
 
 #include "gkd-secret-introspect.h"
 
+const gchar *gkd_secret_introspect_root =
+	"<!DOCTYPE node PUBLIC '-//freedesktop//DTD D-BUS Object Introspection 1.0//EN'\n"
+	"	'http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd'>\n"
+	"<node>\n"
+	"	<node name='org/freedesktop/secrets'/>\n"
+	"</node>\n";
+
 const gchar *gkd_secret_introspect_collection =
 	"<!DOCTYPE node PUBLIC '-//freedesktop//DTD D-BUS Object Introspection 1.0//EN'\n"
 	"	'http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd'>\n"
@@ -53,7 +60,7 @@ const gchar *gkd_secret_introspect_collection =
 	"		</method>\n"
 	"	</interface>\n"
 	"\n"
-	"	<interface name='org.freedesktop.Secrets.Collection'>\n"
+	"	<interface name='org.freedesktop.Secret.Collection'>\n"
 	"		<property name='Items' type='ao' access='read'/>\n"
 	"		<property name='Label' type='s' access='readwrite'/>\n"
 	"		<property name='Locked' type='s' access='read'/>\n"
@@ -84,6 +91,7 @@ const gchar *gkd_secret_introspect_collection =
 	"		</signal>\n"
 	"	</interface>\n"
 	"\n"
+	"<!--@children@-->"
 	"</node>\n";
 
 const gchar *gkd_secret_introspect_item =
@@ -114,7 +122,7 @@ const gchar *gkd_secret_introspect_item =
 	"		</method>\n"
 	"	</interface>\n"
 	"\n"
-	"	<interface name='org.freedesktop.Secrets.Item'>\n"
+	"	<interface name='org.freedesktop.Secret.Item'>\n"
 	"		<property name='Locked' type='b' access='read'/>\n"
 	"		<property name='Attributes' type='a{ss}' access='readwrite'/>\n"
 	"		<property name='Label' type='s' access='readwrite'/>\n"
@@ -146,7 +154,7 @@ const gchar *gkd_secret_introspect_prompt =
 	"		</method>\n"
 	"	</interface>\n"
 	"\n"
-	"	<interface name='org.freedesktop.Secrets.Prompt'>\n"
+	"	<interface name='org.freedesktop.Secret.Prompt'>\n"
 	"		<method name='Prompt'>\n"
 	"			<arg name='window-id' type='s' direction='in'/>\n"
 	"		</method>\n"
@@ -188,7 +196,7 @@ const gchar *gkd_secret_introspect_service =
 	"		</method>\n"
 	"	</interface>\n"
 	"\n"
-	"	<interface name='org.freedesktop.Secrets.Service'>\n"
+	"	<interface name='org.freedesktop.Secret.Service'>\n"
 	"\n"
 	"		<property name='Collections' type='ao' access='read'/>\n"
 	"\n"
@@ -254,6 +262,7 @@ const gchar *gkd_secret_introspect_service =
 	"\n"
 	"	</interface>\n"
 	"\n"
+	"<!--@children@-->"
 	"</node>\n";
 
 const gchar *gkd_secret_introspect_session =
@@ -267,7 +276,7 @@ const gchar *gkd_secret_introspect_session =
 	"		</method>\n"
 	"	</interface>\n"
 	"\n"
-	"	<interface name='org.freedesktop.Secrets.Session'>\n"
+	"	<interface name='org.freedesktop.Secret.Session'>\n"
 	"		<method name='Close'>\n"
 	"		</method>\n"
 	"	</interface>\n"
