@@ -24,7 +24,7 @@
 
 #include "gkd-secret-types.h"
 
-#include "gck/gck.h"
+#include <gck/gck.h>
 
 #include <dbus/dbus.h>
 
@@ -52,6 +52,8 @@ GckSlot*                gkd_secret_service_get_pkcs11_slot         (GkdSecretSer
 GckSession*             gkd_secret_service_get_pkcs11_session      (GkdSecretService *self,
                                                                     const gchar *caller);
 
+GckSession*             gkd_secret_service_internal_pkcs11_session (GkdSecretService *self);
+
 GkdSecretObjects*       gkd_secret_service_get_objects             (GkdSecretService *self);
 
 GkdSecretIndex*         gkd_secret_service_get_index               (GkdSecretService *self);
@@ -66,4 +68,15 @@ void                    gkd_secret_service_close_session           (GkdSecretSer
 void                    gkd_secret_service_send                    (GkdSecretService *self,
                                                                     DBusMessage *message);
 
+const gchar*            gkd_secret_service_get_alias               (GkdSecretService *self,
+                                                                    const gchar *alias);
+
+void                    gkd_secret_service_set_alias               (GkdSecretService *self,
+                                                                    const gchar *alias,
+                                                                    const gchar *identifier);
+
+void                    gkd_secret_service_publish_dispatch        (GkdSecretService *self,
+                                                                    const gchar *caller,
+                                                                    GkdSecretDispatch *object);
+
 #endif /* ___SECRET_SERVICE_H__ */
