@@ -83,6 +83,8 @@ G_DEFINE_TYPE_WITH_CODE (GkdSecretUnlock, gkd_secret_unlock, G_TYPE_OBJECT,
 static guint unique_prompt_number = 0;
 static GQueue unlock_prompt_queue = G_QUEUE_INIT;
 
+EGG_SECURE_DECLARE (secret_unlock);
+
 /* -----------------------------------------------------------------------------
  * INTERNAL
  */
@@ -369,8 +371,9 @@ gkd_secret_unlock_real_dispatch_message (GkdSecretDispatch *base, DBusMessage *m
 	else if (dbus_message_is_method_call (message, SECRET_PROMPT_INTERFACE, "Dismiss"))
 		reply = prompt_method_dismiss (self, message);
 
+	/* org.freedesktop.DBus.Introspectable.Introspect() */
 	else if (dbus_message_has_interface (message, DBUS_INTERFACE_INTROSPECTABLE))
-		return gkd_dbus_introspect_handle (message, gkd_secret_introspect_prompt);
+		return gkd_dbus_introspect_handle (message, gkd_secret_introspect_prompt, NULL);
 
 	return reply;
 }
