@@ -24,7 +24,7 @@
 
 #include "gkd-ssh-agent-private.h"
 
-#include "gck/gck.h"
+#include <gck/gck.h>
 
 #include "pkcs11/pkcs11.h"
 #include "pkcs11/pkcs11i.h"
@@ -39,11 +39,12 @@
 #include <string.h>
 #include <stdio.h>
 
-
 #define V1_LABEL "SSH1 RSA Key"
 
 typedef gboolean (*ObjectForeachFunc) (GckObject *object, gpointer user_data);
 
+EGG_SECURE_DECLARE (ssh_agent_ops);
+
 /* ---------------------------------------------------------------------------- */
 
 
@@ -983,7 +984,7 @@ op_sign_request (GkdSshAgentCall *call)
 	gsize n_data, n_result;
 	guint32 flags;
 	gsize offset;
-	gboolean ret;
+	gboolean ret = FALSE;
 	guint blobpos, sz;
 	guint8 *hash;
 	gulong algo, mech;
